## Copyright (C) 2000 - 2006  SWsoft
## All rights reserved.
## Abstract base class for the engine of payment methods

package HSPC::MT::PM::PluginEngine::OnlineAccount;

use strict;
use SWsoft::protect;

use HSPC::Logger qw(sw_atrace sw_log_warn sw_log_ok);  ##  sw_log sw_die;

use Error qw(:try);
use HSPC::Exception::Plugin::GeneralError;
use HSPC::MT::PM::Constants qw(:pm_status SW_PM_CVV_TEMP);
use HSPC::MT::Billing::Constants;
use HSPC::MT::CCP::Constants qw(:trans_types);

## Inheritance
use HSPC::MT::PM::PluginEngine;

use vars qw{@ISA};
@ISA = qw{HSPC::MT::PM::PluginEngine};

## >> class
sub apply_to_doc {
	my $self = shift;
	my %h = (
		ar_doc           => undef,
		plugin_id	     => undef,
		plugin			 => undef,
		trans_type       => undef,
		paymethod_action => undef,
		billing_details  => undef,
		trans_id         => undef,
		sec_env          => undef,
		@_
	);
	my $trans_type    = $h{trans_type};
	my $paymethod_action = $h{paymethod_action};
	my $doc           = $h{ar_doc} || sw_die("Document is not specified");
	my $plugin_id     = $h{plugin_id};
	my $plugin        = $h{plugin};
	my $is_pending	  = $h{is_pending};
	my $pending_reasone	= $h{pending_reasone};
	my $billing_details = $h{billing_details};
	my $trans_id        = $h{trans_id};

	my $ret;

	## Check status of payment method
	## This should be done only for REAL payment Not REFUND or CAPTURE
	## THIS IS A HACK FOR BAD DESIGN. Should be removed in 3.3.0 Patch 1
	## Filed bug #70394
	if ( !$is_pending && $self->status() != &SW_PM_STATUS_UNCHECKED
		&& $self->status() != &SW_PM_STATUS_OK &&
		$trans_type != &SW_CCP_TRANS_TYPE_REFUND &&
		$trans_type != &SW_CCP_TRANS_TYPE_CAPTURE 
	) {
		$is_pending = 1;
		my %status_string_map = (
			SW_PM_STATUS_EXPIRED  => 'cannot_pay_pm_expired',
			SW_PM_STATUS_ERROR    => 'cannot_pay_pm_error',
			SW_PM_STATUS_DISABLED => 'cannot_pay_pm_disabled',
			SW_PM_STATUS_DELETED  => 'cannot_pay_pm_deleted',
		);

		$ret->{code}     = &SW_PP_PROC_DECLINED;
		$ret->{msg}      = string($status_string_map{$self->status()});
		$ret->{msg_safe} = string('payment_error');
	}

	if (!$plugin && !$plugin_id && !$is_pending) {
		$plugin = HSPC::MT::PP->get_default_plugin(
			paymethod_id => $self->id(),
		);

		unless ($plugin) {
			$ret->{code}     = &SW_PP_PROC_DECLINED;
			$ret->{msg}      = string("cannot_pay_plugins_not_found");
			$ret->{msg_safe} = string("payment_error");
			$is_pending = 1;
		}
	}
	
	if ($is_pending) {
		$self->SUPER::apply_to_doc(
				ar_doc => $doc,
				is_doc_approved => 0,
				is_doc_approved_reason => $ret->{msg} 
						|| $pending_reasone,
				sec_env => $h{sec_env},
				plugin_id => $plugin_id
			);
	} else {
		$self->decrypt;
		$ret = HSPC::MT::PP->pay_by_plugin(
					doc       => $doc,
					paymethod => $self,
					trans_type=> $trans_type,
					plugin_id => $plugin_id,
					plugin    => $plugin,
					vendor_id => $self->vendor_id,
					billing_details => $billing_details,
					trans_id        => $trans_id,
					sec_env         => $h{sec_env},
		       );
	}
	
	if ($paymethod_action eq 'disable')	{
		## Credit Card must be disabled every time if required.
		## See #43137 for details
			$self->status(&SW_PM_STATUS_DISABLED);
			$self->save();
	}
	return $ret;
};

## Check if this paymethod can be applied
## to proveded documents
## Parameters:
##	=>> doc_ids - a reference to an array with document IDs
##	=> document should be checked to be paid recurrently
##	<< a reference to an array with allowed document IDs
sub can_apply_to_docs {
	my $self = shift;
	my %h = (
		doc_ids         => undef,
		pay_recurrently => undef,
		@_
	);

	sw_die("doc_ids was not specified") unless ($h{doc_ids} && scalar @{$h{doc_ids}});

	my $trace = sw_atrace(scalar(@{$h{doc_ids}})." passed");

	## Currently we have the following limitations:
	## 1. Minimal balance of document that can be paid online
	## Recurring Billing limitations:
	## 2. Minimal balance of document that can be paid online recurrently
	## 3. Maximum number of tries have been performed before that moment

	my $request = HSPC::DataRequest->new( cols =>
		[
			{ code => 'id' },
			{ code => 'vendor' },
			{ code => 'doc_balance' },
			{ code => 'doc_type' }
		],
		select_as_hashes => 1
	);

	$request->set_col( code => 'id', for_where => 1, value => $h{doc_ids} );
	my $params;
	push(@$params, 'doc_min_amount');

	if ( $h{pay_recurrently} ) {
		push( @$params, qw(rec_doc_max_attempts rec_doc_max_amount) );
	}

	HSPC::MT::Billing::Ar_doc->load_data( request => $request );

	## Get configuration for all vendors for the following parameters
	my $vendor_confs = HSPC::MT::Billing::Setup->get_params_for_all_vendors(
		params => $params
	);

	my @result_docs;
	foreach my $doc ( @{$request->data} ) {
		## Check doc_type
		next unless ( $doc->{doc_type} eq &DT_IN 
			|| $doc->{doc_type} eq &DT_OR 
			|| $doc->{doc_type} eq &DT_DA
			|| $doc->{doc_type} eq &DT_PD
		);
		if ( $vendor_confs->{$doc->{vendor}}->{doc_min_amount} > 0 &&
			$doc->{doc_balance} < 0 + $vendor_confs->{$doc->{vendor}}->{doc_min_amount}
		) {
			next;
		}

		if ( $h{pay_recurrently} ) {
			if ( $vendor_confs->{$doc->{vendor}}->{rec_doc_max_amount} > 0 &&
				$doc->{doc_balance} > 0 + $vendor_confs->{$doc->{vendor}}->{rec_doc_max_amount}
			) {
				next;
			}
			if ( $vendor_confs->{$doc->{vendor}}->{rec_doc_max_attempts} > 0 ) {
				my $rb_doc = HSPC::MT::BillingFactory->ar_doc_rb_find( id => $doc->{id} );
				if (
					$rb_doc && $rb_doc->id &&
					$rb_doc->trycount > 0 + $vendor_confs->{$doc->{vendor}}->{rec_doc_max_attempts}
				) {
					next;
				}
			}
		}
		## Got so far - save document id for future use
		push(@result_docs, $doc->{id});
	}
	$trace->add( scalar(@result_docs) . " docs allowed" );
	return \@result_docs;
}

sub clear_cvv {
	my $self = shift;
	my $trace = sw_atrace();
	my %h = (
		skip_save => 0,
		@_
	);
	my $skip_save = $h{skip_save};
	my $trace = sw_atrace();

	my $setup = HSPC::MT::Billing::Setup->new(provider_id=>$self->vendor_id);

	## Here we must delete CVV from credit card (VISA requires us not to
	## store this data in the database after the first transaction)
	## Bug #52849
	my $decrypted = 0;
	if ($setup->is_save_cvc == &SW_PM_CVV_TEMP) {
		my $error;

		try {
			$decrypted = $self->decrypt();
		} catch HSPC::Exception::Plugin::SecureKey with {
			## Do nothing
		} catch HSPC::Exception::Plugin::GeneralError with {
			$error = shift;
		} catch HSPC::Exc::Fatal with {
			$error = shift;
		};

		if ( $decrypted ) {
			$self->{keep_secure_code} = 0;
			sw_log_ok("CVV has been marked for removal.");
			$trace->add("CVV has been removed.")
		} else {
			sw_log_warn("Credit Card was not decrypted: $error. There is no possibility to remove CVV.");
			$trace->add("CVV removal skipped.")
		}
	}
	
	if (!$skip_save && $decrypted) {
		$self->save( only_public => 0 );
	}
	
	return $decrypted;
}

sub transaction_declined {
	my $self = shift;
	my $trace = sw_atrace();

	my $decrypted = $self->clear_cvv();
	
	return $self->SUPER::transaction_declined( only_public => !$decrypted );
}

sub transaction_approved {
	my $self = shift;
	my $trace = sw_atrace();

	my $decrypted = $self->clear_cvv();

	return $self->SUPER::transaction_approved( only_public => !$decrypted );
}

1;
